@@ -1,17 +1,20 @@
 import coloredlogs
 import logging
-import pkg_resources
+from importlib.metadata import PackageNotFoundError, version
 
 
 formatter_class = coloredlogs.ColoredFormatter
 
-__version__ = pkg_resources.get_distribution(__name__).version
+try:
+    __version__ = version(__name__)
+except PackageNotFoundError:
+    __version__ = '0.0.0'
 
-__all__ = ['zar']
+__all__ = ['__version__', 'zar']
 
 # create logger
 log = logging.getLogger(__name__)
-log.setLevel(logging.INFO)
+log.level = logging.INFO
 
 # create formatter and add it to the handlers
 _formatter = formatter_class('%(asctime)s|%(name)s-%(levelname)s: %(message)s', datefmt='%Y-%m-%d %H:%M:%S.%f')
@@ -19,10 +22,6 @@ _debug_formatter = logging.Formatter(  # Don't use colored logs
     '%(asctime)s|%(name)s-%(levelname)5s %(threadName)s:%(filename)s:%(lineno)s:%(funcName)s| %(message)s',
 )
 
-# Clear all previously added handlers
-for _ in log.handlers:
-    log.removeHandler(_)
-
 # create console handler
 _ch = logging.StreamHandler()
 _ch.setFormatter(_formatter)
